@@ -35,10 +35,10 @@ of this file. A few common functions are provided in this file (padRow, skipRow,
 useRow).
 
 Requires Python 2.0 or later
-Wizards tested with wxPython 2.2.5/NT 4.0, 2.3.2/Win2000 and Linux/GTK (RedHat 7.x)
+Wizards tested with wxPython 2.2.5/NT 4.0, 2.3.2/Win2000 and Linux/GTK (RedHat 7, 8, 9)
 """
 
-__version__ = '1.4'
+__version__ = '1.4.1'
 
 """
 Bugs/Caveats:
@@ -73,6 +73,16 @@ To do:
 """
 
 # Changelog
+# 1.4.1:
+#   - Several small enhancements, requested by Danny Adair:
+#     - pass record (line) number to errorHandler (importDSV)
+#       WARNING: this is an incompatible but trivial change.  User-defined
+#       error handlers must accomodate the new argument in their signature.
+#       See the demo at the bottom of this file for an example.
+#     - optional newline specifier for exportDSV
+#     - optional stripping of whitespace from data (importDSV)
+#     - optional start, end arguments for importing portions of a file (importDSV)
+#
 # 1.4:
 #   - Fixed small bug in demo (forgotten Destroy()) that caused a hang
 #     when Cancel was pressed.
@@ -419,22 +429,23 @@ def organizeIntoLines(input, textQualifier = '"', limit = None):
 # some common error handlers to pass to importDSV
 # others might do things like log errors to a file.
 # oldrow is the unparsed data, newrow is the parsed data
-def padRow(oldrow, newrow, columns, maxColumns):
+def padRow(linenumber, oldrow, newrow, columns, maxColumns):
     "pads all rows to the same length with empty strings"
     difference = maxColumns - len(newrow)
     return newrow + ([''] * difference)
 
-def skipRow(oldrow, newrow, columns, maxColumns):
+def skipRow(linenumber, oldrow, newrow, columns, maxColumns):
     "skips any inconsistent rows"
     return None
 
-def useRow(oldrow, newrow, columns, maxColumns):
+def useRow(linenumber, oldrow, newrow, columns, maxColumns):
     "returns row unchanged"
     return newrow
 
 # ------------------------------------------------------------------------------
 def importDSV(input, delimiter = ',', textQualifier = '"', columns = 0,
-              updateFunction = None, errorHandler = None):
+              updateFunction = None, errorHandler = None,
+              startline = 0, endline = None, stripall = 0):
     """
     PROTOTYPE:
       importDSV(input, delimiter = ',', textQualifier = '\"', columns = 0,
@@ -452,12 +463,16 @@ def importDSV(input, delimiter = ',', textQualifier = '"', columns = 0,
            - percentDone is an integer between 0 and 100
       - errorHandler is a callback invoked whenever a row has an unexpected number
         of columns. Its prototype is
-           errorHandler(oldrow, newrow, columns, maxColumns)
+           errorHandler(linenumber, oldrow, newrow, columns, maxColumns)
               where
+              - linenumber is where the error occurred in the file
               - oldrow is the unparsed data
               - newrow is the parsed data
               - columns is the expected length of a row
               - maxColumns is the longest row in the data
+      - startline, endline specify how much of the file to parse
+      - stripall causes all data (EVEN QUOTED data) to be mercilessly stripped of whitespace
+      
     RETURNS:
       list of lists of data
     """
@@ -476,11 +491,16 @@ def importDSV(input, delimiter = ',', textQualifier = '"', columns = 0,
 ##        fieldRex = re.compile('(?:[%s]|^)([^%s]*?)(?=[%s]|$)'
 ##                              % (delimiter, delimiter, delimiter), re.S)
 
+    if startline is not None:  # 1.4.1
+        input = input[startline:]
+    if endline is not None:
+        input = input[:endline]
+
     percent = 0.0
     lineno = 0.0
     newdata = []
     maxColumns = 0
-
+                
 ##    for line in input:
 ##        line = line.strip()
 ##        record = fieldRex.findall(line)
@@ -528,9 +548,12 @@ def importDSV(input, delimiter = ',', textQualifier = '"', columns = 0,
             #record = map(lambda x: x.strip(), line.split(delimiter))
             record = map(string.strip, line.split(delimiter))
 
+        if stripall: # 1.4.1
+            record = map(string.strip, record)
+
         newdata.append(record)
         # (end of replacement code)
-        
+
         if updateFunction is not None:
             lineno = lineno + 1.0
             newpercent = int((lineno / len(input)) * 100)
@@ -551,8 +574,8 @@ def importDSV(input, delimiter = ',', textQualifier = '"', columns = 0,
             if errorHandler is None:
                 raise InvalidNumberOfColumns, "Expected %d, got %d" % (columns, length)
             else:
-                newdata[record] = errorHandler(input[record], newdata[record], columns, maxColumns)
-    
+                newdata[record] = errorHandler(record, input[record], newdata[record], columns, maxColumns)
+        
     # remove null values from data
     # newdata = filter(lambda i: i, newdata)
     newdata = filter(None, newdata)
@@ -561,7 +584,7 @@ def importDSV(input, delimiter = ',', textQualifier = '"', columns = 0,
 
 
 # ------------------------------------------------------------------------------
-def exportDSV(input, delimiter = ',', textQualifier = '"', quoteall = 0):
+def exportDSV(input, delimiter = ',', textQualifier = '"', quoteall = 0, newline = '\n'):
     """
     PROTOTYPE:
       exportDSV(input, delimiter = ',', textQualifier = '\"', quoteall = 0)
@@ -597,7 +620,7 @@ def exportDSV(input, delimiter = ',', textQualifier = '"', quoteall = 0):
     data = [delimiter.join(line) for line in data]
     
     # assemble all lines together, separated by newlines
-    data = "\n".join(data)
+    data = newline.join(data)
     return data
 
 if wx is not None:
@@ -1021,9 +1044,9 @@ if __name__ == '__main__':
                     dlg.Destroy()
 
                     errorLog = open('import_error.log', 'a+') 
-                    def logErrors(oldrow, newrow, expectedColumns, maxColumns, file = errorLog):
+                    def logErrors(linenumber, oldrow, newrow, expectedColumns, maxColumns, file = errorLog):
                         # log the bad row to a file
-                        file.write(oldrow + '\n')
+                        file.write("LINE %d: %s\n" % (linenumber, oldrow))
 
                     dlg = ImportWizardDialog(None, -1, 'CSV Import Wizard (v.%s)' % __version__, path)
                     if dlg.ShowModal() == wx.wxID_OK:
